int AllDaysCount;//MAXIMUS: calculates days amount (updating in calendar.c)

#define DEBUG_EXPERIENCE	0					//(int) on 0 no messages are given. on 1 only error, on 2 everything is given.

#define XP_GROUP_PLAYER		"player"			//XP will only be given to the player
#define XP_GROUP_OFFIC		"officers"			//XP will be given to player and officers
#define XP_GROUP_PARTY		"party"				//XP will be given to everyone in the party

#define SKILLS_UP_BONUS			25				// From 10 to 99 - speed of player's skills-increase
#define SKILL_EXPERIENCE_MULTIPLIER	1.0			// FLOAT - Must be a float higher than 0.0 - default is 1.0 - the rate at which the characters gain skill experience: For a 30% increase set to 1.3
#define EXPERIENCE_MULTIPLIER	1.0				// FLOAT - Must be a float higher than 0.0 - default is 1.0 - the rate at which the characters gain experience: For a 30% increase set to 1.3
#define SKILL_NEXT_XP_MULT		1.25			

#define	XP_PERCENTAGE_SHARED	0.20			//(float) percentage which is shared by shared experience
#define	XP_GAIN_OFFIC_ACTIVE	0.80			//(float) percentage which active officer get when XP to all
#define	XP_GAIN_OFFIC_INACTIVE	0.20			//(float) percentage which officer get when XP to all
#define XP_GAIN_COMPANION		0.50			//(float) percentage which companion get when XP to all

#define ADD_SKILLPOINTS_PERLEVEL		2		// INT - The amount of new skill points added each level up for ALL characters (player and officers)
#define ADD_ABILITYPOINTS_PERLEVEL		1		// INT - The amount of new ability points added each level up for ALL characters (player and officers)

#define IMPORTANCE_BONUS_GAIN			25		// INT - The multiplier for the importance if a bonus is present for this skill
#define IMPORTANCE_SKILL_GAIN			30		// INT - The multiplier for each level the character has already in this skill 
#define IMPORTANCE_BOOST				30		// INT - The multiplier for the importance set in officers.c
//----------------------------------------------------------------------------------------------//
//Functions to handle XP gain																	//
//----------------------------------------------------------------------------------------------//

//Levis' New functions to handle everything

//Called for all instances
bool AddXP(ref chref, string expName, int _exp, string group)
{
	bool LevelUp = false;
	//First catch some weird cases which shouldn't happen in the first place. let log if they do happen.
	if(!CheckAttribute(chref,"index")) { return; if(DEBUG_EXPERIENCE>0) Trace("XP ERROR: Character "+chref.id+" has no index");}
	if(sti(chref.index)<0) { return; if(DEBUG_EXPERIENCE>0){ Trace("XP ERROR: Character "+chref.id+" has index smaller then 0");} }
	if(_exp <= 0) 
	{ 
		_exp = -(_exp); 
		if(DEBUG_EXPERIENCE>0) { Trace("XP ERROR: NEGATIVE OR ZERO XP");} 
	}
	if(_exp == 0) return false;
	
	//Check for shared experience
	bool SharedXP = GetOfficersPerkUsing(chref,"SharedExperience");
	
	if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Called AddXP for "+chref.id+" with skill: "+expName+" and xp: "+_exp+" and group: "+group);
	if(DEBUG_EXPERIENCE>1) { if(SharedXP) Trace("XP LOG: SharedXP is active"); }
	
	//First handle the character which gets the XP
	float xpmult = GetXPmult(expName);
	int xp = makeint(makefloat(_exp)*xpmult);
	if(AddXPtoChar(chref, expName, xp)) LevelUp = true;
	
	//Now handle the officers
	int cn;
	if(SharedXP || group == XP_GROUP_OFFIC || group == XP_GROUP_PARTY)
	{
		if(FindCaptainIndex(chref) >= 0)
		{
			//If this character is part of a party we want to have the whole party.
			ref captain = GetCharacter(FindCaptainIndex(chref));
			if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Checking officers for captain "+captain.id);
			//Check if the captain has got the XP already
			if(captain.index != chref.index)
			{
				xpmult = XP_GAIN_OFFIC_ACTIVE;
				if(SharedXP) xpmult += XP_PERCENTAGE_SHARED;
				xp = makeint(_exp*xpmult*EXPERIENCE_MULTIPLIER);
				if(AddXPtoChar(captain, expName, xp)) LevelUp = true;
			}
			if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Start checking officers");
			for(int i=0; i < GetPassengersQuantity(captain); i++)
			{
				cn = GetPassenger(chref, i);
				//Filter the original character and prisoned characters
				if(cn < 0 || cn == chref.index) continue;
				if(CheckAttribute(Characters[cn],"prisoned") && sti(Characters[cn].prisoned)) continue;
				//Set the percentage of xp given.
				xpmult = 0;
				if(SharedXP) xpmult = XP_PERCENTAGE_SHARED;
				if(group == XP_GROUP_OFFIC)
				{
					if (IsOfficer(Characters[cn]))
					{
						xpmult += XP_GAIN_OFFIC_ACTIVE;
					}
					else
					{
						xpmult += XP_GAIN_OFFIC_INACTIVE;
					}
				}
				xp = makeint(_exp*xpmult*EXPERIENCE_MULTIPLIER);
				//Add the experience
				if(AddXPtoChar(&Characters[cn], expName, xp)) LevelUp = true;
			}
		}
	}
	
	//Now handle the companions
	if(SharedXP || group == XP_GROUP_PARTY)
	{
		if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Checking companions");
		if(GetCompanionQuantity(chref) > 1)
		{
			for(i=1; i < 4; i++)
			{
				cn = GetCompanionIndex(chref, i);
				//Filter the original character and prisoned characters
				if(cn < 0 || cn == chref.index) continue;
				if(CheckAttribute(Characters[cn],"prisoned") && sti(Characters[cn].prisoned)) continue;
				//Set the percentage of xp given.
				xpmult = 0;
				if(SharedXP) xpmult = XP_PERCENTAGE_SHARED;
				if(group == XP_GROUP_PARTY) xpmult += XP_GAIN_COMPANION;
				xp = makeint(_exp*xpmult*EXPERIENCE_MULTIPLIER);
				//The companion might have officers also so call self to manage this but with less XP already.
				if(AddXP(&Characters[cn], expName, xp, XP_GROUP_OFFIC)) LevelUp = true;;
			}
		}
	}
	//returns true if there is a levelup somewhere
	return LevelUp;
}

//Handle for a single character
bool AddXPtoChar(ref chref, string expName, int _exp)
{
	bool LevelUp = false;
	
	//Check if everything is okay
	if(!CheckAttribute(chref,"skillsetup")) 
	{
		CheckCharacterSetup(chref);
	}
	else
	{
		//In case the officertype is changed, we should apply the importances again
		if(chref.skillsetup != chref.quest.officerType)
		{
			if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Offictype for "+chref.id+" is changed so reapply the importances.");
			SetCharSkillImportance(chref,1);
		}
	}

	//Now adjust the experience for difficulty and such

	int realExp = makeint(1.0 * _exp * (1.0-((GetDifficulty()-1.0) / 4.0)));

	if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Called AddXPtoChar for "+chref.id+" with skill: "+expName+" and xp: "+realExp+ " CurXP: "+chref.Experience);
	
	//First add the skillxp so we can use the increase for perk assignment when needed.
	//If we are using the auto skill system let's assign the skill points:
	//Also use it for character which aren't part of the characters party so they are set up right
	if(AUTO_SKILL_SYSTEM || !bAllies(chref))
	{
		if(expName != "")
		{
			AddXPtoSkill(chref, expName, realExp);
			//If we have a specific skill increase we should call the update importace function
			UpdateSkillImportance(chref, expName, realExp);
		}
		else
		{
			AddXPtoAllSkills(chref, realExp);
		}
	}
	
	//Now add the overal experience
	//Add the experience to the character.

	int nextExp = CalculateExperienceFromRank(sti(chref.rank)+1);
	int curExp = sti(chref.Experience) + realExp;
	float mhp = 0;

	chref.Experience = curExp;
	
	//Check if we go level up
	bool firstlevelup = true;
	while (curExp >= nextExp)
	{
		//Increase the rank of the character
		chref.rank = sti(chref.rank) + 1;
		//Calculate new nextExp
		nextExp = CalculateExperienceFromRank(sti(chref.rank)+1);
		//Set new HP
		ResetHP(chref);
		if(CheckAttribute(chref,"officer")) mhp = LAi_GetCharacterMaxHP(chref) + makeint(11-GetDifficulty());
		else mhp = LAi_GetCharacterMaxHP(chref) + 5.0;
		LAi_SetHP(chref,LAi_GetCharacterHP(chref),mhp);
		//Add freepoints for perks and skills
		chref.perks.FreePoints = makeint(chref.perks.FreePoints) + ADD_ABILITYPOINTS_PERLEVEL;
		if(!AUTO_SKILL_SYSTEM) chref.skill.freeskill = sti(chref.skill.freeskill) + ADD_SKILLPOINTS_PERLEVEL;
		//Set new officer price
		if(CheckAttribute(chref,"quest.officerprice")) chref.quest.officerprice = sti(chref.quest.officerprice) + 500 * OPRICE_LEVEL_MULT;
		//Set return value
		LevelUp = true;
		//Display the levelup
		if(IsInParty(GetMainCharacterIndex(), sti(chref.index)))
		{
			Log_SetStringToLog(
				GetMySimpleName(chref) + " "
				+ TranslateString("","has gained a new level! (Now level")
				+ " " + chref.rank + ")");
			PlayStereoSound("interface\new_level.wav");
			AddMsgToCharacter(chref,MSGICON_LEVELUP);
		}
		else
		{
			//This character has noone to choose his perks so we must do it for him.
			if(firstlevelup) { PrepareSelectPerksForCharacter(chref); firstlevelup = false;}
			SelectPerksForCharacter(chref);
		}
		// LDH 01May09
		if(DEBUG_EXPERIENCE>1) Trace("XP LOG: "+GetMySimpleName(chref) + " " + TranslateString("","has gained a new level! (Now level") + " " + chref.rank + ")");
	}
	DeleteAttribute(chref,"PerkTypes");
	return LevelUp;
}

//Get xp multiplier for different skills
float GetXPmult(string Skillname)
{
	float expMult=1;
	switch (Skillname)
	{
		case "Leadership":	expMult = 5.5; break;	
		case "Fencing":		expMult = 5; break;
		case "Sailing":		expMult =  2; break;
		case "Accuracy":	expMult =  1.5; break;
		case "Cannons":		expMult =  0.5; break;
		case "Repair":		expMult = 50; break;
		case "Defence":		expMult =  6; break;
		case "Sneak":		expMult =  4.5; break;
		case "Grappling":	expMult =  4; break;
		case "Commerce": 	expMult =  0.75; break;
	}
	return expMult*EXPERIENCE_MULTIPLIER;
}

//Add XP to all skills
bool AddXPtoAllSkills(ref chref, int skillExp)
{
	if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Called AddXPtoAllSkills with Exp: "+skillExp);
	bool LevelUp = false;
	//Start the loop
	aref skills; makearef(skills,chref.skill);
	string skill;
	int numskills = GetAttributesNum(skills);
	int importance = 0;
	float mult;
	float xp = 0;
	for(int n = 0; n < numskills; n++)
	{
		skill = GetAttributeName(GetAttributeN(skills, n));
		if(skill == "freeskill") continue;
		importance = sti(chref.skill.(skill).importance);
		//We got a total amount of XP to spend. We divide this over the different skills based on the importance of the skill
		mult = makefloat(importance) / stf(chref.skillimportancetotal);
		xp = makefloat(skillExp) * mult;
		if(AddXPtoSkill(chref, skill, makeint(xp))) LevelUp = true;
	}
	return LevelUp;
}

//Adding XP to a skill
bool AddXPtoSkill(ref chref, string expName, int skillExp)
{
	bool LevelUp = false;
	int chrSkillExp = sti(chref.Experience.(expName));
	
	//Add the bonus to the experience
	if (CheckAttribute(chref,"skill."+expName+".bonus")) skillExp = makeint(skillExp * stf(chref.skill.(expName).bonus));
	
	// Add the multiplier
	skillExp = makeint(skillExp * SKILL_EXPERIENCE_MULTIPLIER);

	// Add to currSkill
	int curSkill = chrSkillExp + skillExp;
	
	if(DEBUG_EXPERIENCE>1) Trace("XP LOG: AddXP to skill: "+expName+" Old Skill Exp: "+chrSkillExp+" increase with: "+makeint(skillExp) +" needed for next level: "+CalculateSkillExperienceFromRank(sti(chref.skill.(expName))+1));

	int nextSkill = CalculateSkillExperienceFromRank(sti(chref.skill.(expName))+1);
	while (curSkill >= nextSkill)
	{
		LevelUp = true;
		//Substract the xp needed to level up.
		curSkill = curSkill-nextSkill;
		//Add the level
		if(sti(chref.skill.(expName)) >= SKILL_MAX)
		{
			//We don't need any more XP on this skill
			if(sti(chref.skill.(expName).importance) > 0)
			{
				chref.skillimportancetotal = sti(chref.skillimportancetotal) - sti(chref.skill.(expName).importance);
				chref.skill.(expName).importance = 0;
				if(DEBUG_EXPERIENCE>1) Trace("Importance for "+expName+" is set to 0 because skill is maxed out");
			}
		}
		else
		{
			chref.skill.(expName) = sti(chref.skill.(expName)) + 1;
			if (iRealismMode == 0 && LogsToggle >= LOG_NORMAL && IsInParty(GetMainCharacterIndex(), sti(chref.index))) { // KK
				Log_SetStringToLog(GetMySimpleName(chref) + " " + TranslateString("","Gained") + " " + XI_ConvertString("skill_level") + " " + XI_ConvertString(expName) + " - " + sti(chref.skill.(expName)));
			}
		}
		//Calculate next needed skill exp
		nextSkill = CalculateSkillExperienceFromRank(sti(chref.skill.(expName))+1);
	}
	//Change amount of xp on char
	chref.Experience.(expName) = curSkill;
	return LevelUp;
}

int CalculateSkillExperienceFromRank(int rank)
{
	//makeint(((CalculateExperienceFromRank(rank)-CalculateExperienceFromRank(rank-1))*SKILL_NEXT_XP_MULT) / ADD_SKILLPOINTS_PERLEVEL
	int maxexp = CalculateExperienceFromRank(makeint((10 * SKILL_MAX) / ADD_SKILLPOINTS_PERLEVEL)); //Take the XP needed for level all skillpoints are used
	//We got 10 skills each going to SKILL_MAX we need to know how !SKILL_MAX so use a forloop.
	int amskills = 0;
	for(int n = 1; n < SKILL_MAX; n++) {amskills += pow(n,2);}
	return makeint((maxexp/10) / amskills * pow((rank-1),2));
}

void UpdateSkillImportance(ref chref, string expName, int skillExp)
{
	if(sti(chref.skill.(expName)) >= SKILL_MAX) return;
	//small bursts of experience will probably increase your importance more then one large burst.
	//At least thats what this function is aiming for.
	int nextSkill = makeint(CalculateExperienceFromRank(sti(chref.skill.(expName))+1) / ADD_SKILLPOINTS_PERLEVEL);
	//We want to know the percentage of experience we gathered for the nextskill
	//We take a random number from the skillExp to have randomization in it and favor small bursts
	float percentage = rand(skillExp) / nextSkill * 100;
	//if the percentage is over 50% we should increase the importance by default. so we multiply the percentage times 2
	//We take a random number from this percentage and see if we are lucky.
	int chance = makeint(percentage * 2);
	if(DEBUG_EXPERIENCE>1) Trace("XP LOG: chance of increasing importance: "+chance+" for skill: "+expName);
	if(rand(100)<chance)
	{
		//Very low importance should increase fast so you can level this skill easier.
		int importance = sti(chref.skill.(expName).importance);
		int inc = 0;
		if(importance < 25) inc = 3;
		if(importance < 50) inc = 1;
		if(importance < 75) inc = 2;
		if(importance < 75) inc = 3;
		chref.skill.(expName).importance = importance + inc;
		chref.skillimportancetotal = sti(chref.skillimportancetotal) + inc;
	}
	//We shouldn't add an max because when playing longer all the importances will grow. At one moment they will all be capped to the max then.
	//This way they would eventually all increase lineair.
}

//Adding skillpriorities to a character
void SetCharSkillImportance(ref chref, int SkillVariation)
{
	if(DEBUG_EXPERIENCE>1) Trace("XP LOG: add skill importance for character: "+chref.id);
	//Check if there is an offictype assigned
	if(!CheckAttribute(chref,"quest.officertype"))
	{
		chref.quest.officertype = OFFIC_TYPE_RANDCHAR;
		if(IsMainCharacter(chref)) chref.quest.officertype = OFFIC_TYPE_CIVILIAN;
		if(DEBUG_EXPERIENCE>0) Trace("XP ERROR: No officertype assigned to: "+chref.id+" now set to: "+chref.quest.officertype);
	}
	string offictype = chref.quest.officertype;
	//Start the loop
	ref pchar = GetMainCharacter();
	//We use the skill list in maincharacter to loop trough so we know its always accurate
	aref skills; makearef(skills,pchar.skill);
	string skill;
	int importance = 0;
	int numskills = GetAttributesNum(skills);
	int total = 0;
	for(int n = 0; n < numskills; n++)
	{
		skill = GetAttributeName(GetAttributeN(skills, n));
		if(skill == "freeskill") continue;
		//Check if a level has been set already
		if(!CheckAttribute(chref,"skill."+skill)) chref.skill.(skill) = 1;
		//Get the importance for the skill
		if(CheckAttribute(OfficerTypes,offictype+".skills."+skill+".importance"))
		{
			importance = sti(OfficerTypes.(offictype).skills.(skill).importance)*IMPORTANCE_BOOST+rand(SkillVariation*10);
		}
		else
		{
			if(DEBUG_EXPERIENCE>1) Trace("XP LOG: No importace assiged to: "+skill+" for offictype: "+offictype+" so use default");
			importance = 10+rand(SkillVariation*10);
		}
		//When we have a bonus for the skill increase importace
		if(CheckAttribute(chref,"skill."+skill+".bonus")) 
		{
			if(stf(chref.skill.(skill).bonus) > 1)
			{
				importance += IMPORTANCE_BONUS_GAIN+rand(SkillVariation*10);
				if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Bonus found so importance increased");
			}
		}
		//If the level is higher the 1 for this skill it should also get extra importance
		if(sti(chref.skill.(skill)) > 1)
		{
			importance += IMPORTANCE_SKILL_GAIN*sti(chref.skill.(skill));
			if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Character has level in skill so add importance");
		}
		if(DEBUG_EXPERIENCE>1) Trace("XP LOG:  Set skill "+skill+" to importance: "+importance+" for "+chref.id);
		//Set the importance
		chref.skill.(skill).importance = importance;
		total += importance;
		//Set experience for skill in case it isn't set yet
		if(!CheckAttribute(chref,"Experience."+skill)) chref.Experience.(skill) = 0;
	}
	//Set this so we know when it's set for someone.
	chref.skillimportancetotal = total;
	chref.skillsetup = chref.quest.officerType;
}

void CheckCharacterSetup(ref chref)
{
	if (!CheckAttribute(chref,"id")) {chref.id = "without_id"; }
	if (!CheckAttribute(chref,"rank")) {chref.rank = 1; }
	if (!CheckAttribute(chref,"Experience")) {chref.Experience = 1; }
	if (!CheckAttribute(chref,"perks.FreePoints")){chref.perks.FreePoints = 0; } 
	if (!CheckAttribute(chref,"skill.freeskill")){chref.skill.freeskill = 0; } 
	if(!CheckAttribute(chref,"quest.officertype"))
	{
		if(CheckAttribute(chref,"isOfficer"))
		{ 
			if(chref.isOfficer == "1") chref.quest.officertype = GetRandomOfficerType();
		}
		else
		{
			if(CheckAttribute(chref,"issoldier"))
			{ 
				if(chref.issoldier == "1") chref.quest.officertype = OFFIC_TYPE_GUARD;
			}
			else
			{
				if(CheckAttribute(chref,"isMerchant"))
				{ 
					if(chref.isMerchant == "1") chref.quest.officertype = OFFIC_TYPE_SHOPKEEPER;
				}
				else
				{
					chref.quest.officerType = GetRandomNPCType();
				}
			}
		}
	}
	if (!CheckAttribute(chref,"skillsetup")) SetCharSkillImportance(chref, 1);
	if (!CheckAttribute(chref,"Money")) {
        chref.Money = GetRandCharMoney(chref, makeint(Rand(15)+10));
    } else {
        if (sti(chref.Money) == 0) chref.Money = GetRandCharMoney(chref, makeint(Rand(15)+10));
    }
}

void PrepareSelectPerksForCharacter(ref chref)
{
	if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Prepare SelectPerksForCharacter for: "+chref.id);
	//Let's check the different perktypes and see what our character can pick
	//We use the skill list in maincharacter to loop trough so we know its always accurate
	int numperktypes = GetAttributesNum(PerkTypes);
	aref perktype;
	int total = 0;
	int numskills = 0;
	int weight = 0;
	int sumskill = 0;
	int highest = -1;
	int hval = 0;
	string skillname, typename;
	for(int n = 0; n < numperktypes; n++)
	{
		typename = GetAttributeName(GetAttributeN(PerkTypes, n));
		makearef(perktype,PerkTypes.(typename).Skill);
		//A perktype has serveral contributing skills. They are weighted and when added together with theire
		//weighting they can be max 100. We use this number to determine which skilltype will be picked.
		sumskill = 0;
		numskills = GetAttributesNum(perktype);
		for(int i = 0; i < numskills; i++)
		{
			weight = sti(GetAttributeValue(GetAttributeN(perktype, i)));
			skillname = GetAttributeName(GetAttributeN(perktype, i));
			sumskill += weight*sti(chref.skill.(skillname))
		}
		chref.PerkTypes.(typename) = sumskill;
		if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Perktype: "+typename+" has a score of: "+sumskill);
		total += sumskill;
		if(sumskill > hval) {highest = n; hval=sumskill;}
	}
	if(DEBUG_EXPERIENCE>1) Trace("XP LOG: There is a total of: "+total);
	chref.PerkTypes = total;
	chref.PerkTypes.highest = highest;
}

bool SelectPerksForCharacter(ref chref)
{
	string typename;
	int numperktypes = GetAttributesNum(PerkTypes);
	if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Called SelectPerksForCharacter for: "+chref.id);
	//First check if we have a usable officer else we only have to try the combat perks
	string offictype = chref.quest.officertype;
	if(CheckAttribute(OfficerTypes,offictype+".perks"))
	{
		
		//First we try to add a perk to the highest PerkType
		typename = GetAttributeName(GetAttributeN(PerkTypes, sti(chref.PerkTypes.highest)));
		if(SelectPerkForType(typename, chref))
		{
			return true;
		}
		else
		{
			//Let's try again a few more times and see if we can find a nice perk.
			int rnum = 0;
			for(int n = 0; n < 2; n++)
			{
				if(DEBUG_EXPERIENCE>1) Trace("XP LOG: attempt "+(n+2)+" to find a perk");
				//Let's find a random perktype. By taking a random of the total and checking it agains the weights we should still
				//Take the character into account
				rnum = rand(sti(chref.PerkTypes));
				for(int i = 0; i < numperktypes; i++)
				{
					typename = GetAttributeName(GetAttributeN(PerkTypes, n));
					if(rnum < sti(chref.PerkTypes.(typename)))
					{
						if(SelectPerkForType(typename, chref)) 
						{
							return true;
						}
						break;
					}
					else
					{
						rnum -= sti(chref.PerkTypes.(typename));
					}
				}
			}
		}
	}
	else
	{
		//Use Type Combat
		if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Officertype doesn't have any perks assigned, so use COMBAT instead");
		if(SelectPerkForType(PERK_TYPE_COMBAT, chref))
		{
			return true;
		}
	}
	return false;
}

bool SelectPerkForType(string type, ref chref)
{
	if(DEBUG_EXPERIENCE>1) Trace("XP LOG: called SelectPerkForType for type: "+type);
	aref perks; makearef(perks,PerkTypes.(type).Perks);
	//Let's loop trough the perks and see if there is any we can assign.
	int numperks = GetAttributesNum(perks);
	string perkname;
	string offictype = chref.quest.officertype;
	if(type == PERK_TYPE_COMBAT) offictype = "Combat";
	int score = 0;
	int highest = -1;
	int hval = 0;
	for(int n = 0; n < numperks; n++)
	{
		perkname = GetAttributeName(GetAttributeN(perks, n));
		//See if this perk is avaible for this offictype
		if(CheckAttribute(OfficerTypes,offictype+".perks."+perkname))
		{
			//We keep an score for each perk, the better the score the higher the chance it will be chose.
			if(isPerkAssignableNoLock(chref, perkname, sti(chref.perks.FreePoints)))
			{
				//A higher cost means a more interesting perk normally so if we can affort it we should take it.
				score = sti(perks.(perkname));
				//Check if the main character has the perk already unlocked
				if(CheckCharacterPerkLocked(chref, perkname))
				{
					//If the perk is locked there is a chance this will be unlocked. At level 25 all will be unlocked
					if(rand(50)<pow(sti(chref.rank),1.2))
					{
						//If the perk is unlocked this is good so we add something to the score
						score += 1;
					}
				}
				if(CheckAttribute(perks,perkname+".rank"))
				{
					if(sti(chref.rank) >= sti(perks.(perkname).rank))
					{
						//We passed the required rank for this perk so that makes it more important
						score += 1;
					}
				}
				if(CheckAttribute(perks,perkname+".unlocks"))
				{
					//This perk unlocks new perks so that makes it extra important
					score += 2;
				}
				if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Perk: "+perkname+" is avaible and has a score of: "+score);
				if(score > highest){ highest = n; hval = score;}
			}
		}
	}
	if(highest >= 0)
	{
		int chance = 100;
		if(hval <2) chance = 20; //If the score is only 1 we have a 20% chance of taking this perk. There might be something better or we should wait till something with more points comes along
		if(rand(100)<chance)
		{
			perkname = GetAttributeName(GetAttributeN(perks, highest));
			if(CheckCharacterPerkLocked(chref, perkname))
			{
				UnlockPerkCharacter(chref, perkname); 
				if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Unlocked perk: "+perkname+" for: "+chref.id);
			}
			chref.perks.list.(perkname) = 1;
			chref.perks.FreePoints = sti(chref.perks.FreePoints) - GetPerkCost(perkname);
			DeleteAttribute(chref,"ContribList");
			if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Assigned perk: "+perkname+" to: "+chref.id);
			return true;
		}
	}
	return false;
}

void InitCharacterSkills(ref chref)
{
	if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Start InitCharacterSkills");
	CheckCharacterSetup(chref); //First check if everything is there.
	if(CheckAttribute(chref,"questchar")) return; //We don't alter questcharacters
	int rank = sti(chref.rank);
	int skillrank = ResetSkillsandPerks(chref);
	if(DEBUG_EXPERIENCE>1)trace("XP LOG: Rank Check "+chref.id+" rank: "+rank+" skillrank: "+skillrank);
	if(skillrank > rank) rank = skillrank;
	if(rank>1)
	{
		float mult = 1.0 / (1.0-((GetDifficulty()-1.0) / 4.0)); //Temp fix untill i find something better. I think I need to add a switch to the AddXP function...
		AddXP(chref, "", CalculateExperienceFromRank(rank)*mult, XP_GROUP_PLAYER);
	}
}

int ResetSkillsandPerks(ref chref)
{
	//Reset the Character to level 1
	if(DEBUG_EXPERIENCE>1) trace("XP LOG: Character "+chref.id+" has been reset to 0"); //Show these always
	aref skills; makearef(skills,chref.skill);
	string skill;
	int numskills = GetAttributesNum(skills);
	int skillrank = 0;
	for(int n = 0; n < numskills; n++)
	{
		skill = GetAttributeName(GetAttributeN(skills, n));
		if(skill == "freeskill") continue;
		skillrank += (sti(chref.skill.(skill))-1);
		chref.skill.(skill) = 1;
	}
	chref.perks.FreePoints = 0;
	chref.skill.freeskill = 0;
	chref.Experience = 0;
	ResetHP(chref);
	DeleteAttribute(chref,"perks");
	DeleteAttribute(chref,"ContribList");
	chref.rank = 1;
	return makeint(skillrank/ADD_SKILLPOINTS_PERLEVEL);
}

//Add XP to party

// this will add experience to all passengers, including officers and companions
void AddPartyExpChar(ref chref, string expName, int _exp)
{
	AddXP(chref, expName, _exp, XP_GROUP_PARTY);
	/*if(!CheckAttribute(chref,"index")) { return; }
	if(sti(chref.index)<0) { return; }
	int cn,cn2,i,j, myindex;
	myindex = sti(chref.index);
	float sxmult;

	// LDH - 01Apr09 -->
	// if any officer has SharedExperience, everyone has it
	sxmult = (1.0 + GetOfficersPerkUsing(chref,"SharedExperience")) / 4.0;	// this will be 0.25 or 0.5

	AddCharacterExpNSChar(chref, expName, _exp);	// this adds to the original character
	if(IsOfficer(chref) || IsCompanion(chref))		// this includes main character
	{
		if (myindex != GetMainCharacterIndex())		// if chref not main character
		{
			chref = GetMainCharacter();				// this sets original character to main character for later calcs
			AddCharacterExpNSChar(chref, expName, _exp * sxmult);	// this adds to the main characer if original was not main character
		}
	}

	if (sti(chref.index) != GetMainCharacterIndex()) return;	// Don't continue process on characters who are not our officers or oompanions
	// LDH <--

	for(i=0; i < GetPassengersQuantity(chref); i++)		// does not include main character
	{
		cn = GetPassenger(chref, i);
		if(cn == -1 || cn == myindex) continue;			// bypass the original character
		if(CheckAttribute(Characters[cn],"prisoned") && sti(Characters[cn].prisoned)) continue;
		if (IsOfficer(Characters[cn]))
		{
			AddCharacterExpNSChar(&Characters[cn], expName, _exp * sxmult);	// // LDH - 01Apr09 active officers get full experience here, 0.25 or 0.5 times experience
		}
		else
		{
			AddCharacterExpNSChar(&Characters[cn], expName, _exp * 0.5 * sxmult);	// inactive officers get half experience
		}
	}
	if(GetCompanionQuantity(chref) > 1)
	{
		for(i=1; i < 4; i++)						// does not include main character
		{
			cn = GetCompanionIndex(chref, i);
			if(cn == -1) continue;
			for(j=0; j<4; j++)
			{
				cn2 = GetOfficersIndex(&Characters[cn], j);
				if(cn2 == -1 || cn2 == myindex) continue;	// bypass the original character
				AddCharacterExpNSChar(&Characters[cn2], expName, _exp * sxmult);	// // LDH - 01Apr09 companions get full experience here, 0.25 or 0.5 times experience
			}
		}
	}*/
}

// NK -->
void AddPartyExp(ref chref, int _exp)
{
	AddXP(chref, "", _exp, XP_GROUP_PARTY);
	/*int cn,cn2,i,j, myindex;
	myindex = sti(chref.index);
	float sxmult = 1.0 + GetOfficersPerkUsing(chref,"SharedExperience");
	AddCharacterExpNS(chref, _exp);
	if(IsOfficer(chref) || IsCompanion(chref)) { chref = GetMainCharacter(); AddCharacterExpNS(chref, _exp); }
	for(i=0; i < GetPassengersQuantity(chref); i++)
	{
		cn = GetPassenger(chref, i);
		if(cn == -1 || cn == myindex) continue;
		if(CheckAttribute(Characters[cn],"prisoned") && sti(Characters[cn].prisoned)) continue;
		if(IsOfficer(Characters[cn])) {AddCharacterExpNS(&Characters[cn], _exp);}
		else {AddCharacterExpNS(&Characters[cn], _exp * 0.5 * sxmult);}
	}
	if(GetCompanionQuantity(chref) > 1)
	{
		for(i=1; i < 4; i++)
		{
			cn = GetCompanionIndex(chref, i);
			if(cn == -1) continue;
			for(j=0; j<4; j++)
			{
				cn2 = GetOfficersIndex(&Characters[cn], j);
				if(cn2 == -1 || cn2 == myindex) continue;
				AddCharacterExpNS(&Characters[cn2], _exp);
			}
		}
	}*/
}
// NK <--

//Add xp to Officers and Mainchar

// this will add experience to all officers, but not other passengers or companions
int AddCharacterExpChar(ref chref, string expName, int _exp)
{
	return makeint(AddXP(chref, expName, _exp, XP_GROUP_OFFIC));
	/*if(!CheckAttribute(chref,"index")) { return 0; }
	if(sti(chref.index)<0) { return 0; }

	int i, cn, myindex;
	myindex = sti(chref.index);
	AddCharacterExpNSChar(chref, expName, _exp);

	// LDH 01Apr09
	float sxmult;
	sxmult = (1.0 + GetOfficersPerkUsing(chref,"SharedExperience")) / 4.0;	// will be 0.25 or 0.5

	if (IsOfficer(chref))		// this includes main character
	{
		ref rchr = GetMainCharacter();
		for(i=0; i < 4; i++)			// this includes main character
		{
			cn = GetOfficersIndex(rchr, i);
			if(cn == -1) { continue; }
			if(cn == myindex) { continue; }			// bypass the original character
			AddCharacterExpNSChar(&Characters[cn], expName, makeint(makefloat(_exp) * sxmult));	// officers get 0.25 or 0.5 times experience
		}
	}
	if(!CheckAttribute(chref,"Experience")) chref.Experience = 1;
	return sti(chref.Experience);*/
}

//Add XP to character

// this adds experience to an individual character
int AddCharacterExpNSChar(ref chref, string expName, int _exp)
{
	return makeint(AddXP(chref, expName, _exp, XP_GROUP_PLAYER));
	/*if(!CheckAttribute(chref,"index")) return 0;
	if(sti(chref.index)<0) return 0;
	
	_exp = _exp * EXPERIENCE_MULTIPLIER;		// changed by El Rapido to make it configurable in InternalSettings.h
	
	if(_exp < 0) _exp = -(_exp);	// LDH these are probably coming from store code when you autobuy goods - 02Apr09
	if(_exp < 1) return 0;
		
	if(CheckAttribute(chref,"tmpExp")) DeleteAttribute(chref,"tmpExp");

	//==========================================================
	// Make sure our character has all the necessary attributes
	//==========================================================

	if (!CheckAttribute(chref,"id")) { chref.id = "without_id"; }
	if (!CheckAttribute(chref,"rank")) { chref.rank = 1; }
	if (!CheckAttribute(chref,"Experience")) chref.Experience = 1;
	if (!CheckAttribute(chref,"Experience."+expName))
	{
		chref.Experience.(expName) = makeint(sti(Rand(19)+1)*1000);
	}
	if (!CheckAttribute(chref,"skill."+expName) || sti(chref.skill.(expName)) == 0)
	{
		chref.skill.(expName) = 1;
	}

	//==================================
	// Adjust experience for difficulty
	//==================================
	// If we aren't using Auto skill system there is no adjustment for difficulty, and all calls to AddPartyExp get double experience

	float realExpFloat;

	if(bAllies(chref))		// *** this includes main character ***
	{
		realExpFloat = 1.0 * _exp * (1.0-((GetDifficulty()-1.0) / 4.0));	 // 1.0, 0.75, 0.5, 0.25
	}
	else
	{
		// LDH - 02Apr09
		realExpFloat = 4.0/3.0 * _exp * (1.0-((GetDifficulty()-1.0) / 4.0)); // 1.33, 1.0, 0.67, 0.33
		if(CheckAttribute(chref,"chr_ai.type"))
		{
			if(chref.chr_ai.type != LAI_TYPE_BUILDING)
			{
				if(!CheckAttribute(chref,"resorted"))
				{
					if(CheckAttribute(chref, "perks.FreePoints") == true && sti(chref.perks.FreePoints)>0) // KK
					{
						ResortFreePerks(chref);
						chref.resorted = true;
					}
				}
			}
		}
	}

	int realExp;
	realExp = makeint(realExpFloat);	// LDH 01Apr09

	//=====================================
	// Add the experience to the character
	//=====================================

	/*int nextExp = CalculateExperienceFromRank(sti(chref.rank)+1);
	int curExp = sti(chref.Experience) + realExp;

	chref.Experience = curExp;

	if (LogsToggle == LOG_VERBOSE && IsInParty(GetMainCharacterIndex(), sti(chref.index)) == true && realExp > 10 * makeint(chref.rank) && realExp > 25 && !CheckAttribute(chref,"donotshowXP")) // KK
	{
		Log_SetStringToLog(GetMySimpleName(chref) + " " + TranslateString("","Gained") + " " + realExp + " " + XI_ConvertString("XP"));
	}*/
	
	/*int retVal = AddCharacterExpNS(chref,realExp); //Levis, use function to make sure everything is at one place.

	//===========================================================================================================
	// At this point the experience has been applied to the character and everything in this section is skilling
	//===========================================================================================================

	// 10000 skillExp points make one skill point

	int skillExp = realExp;		// LDH added 02Apr09
	
	if (skillExp > 5000) skillExp = 5000;		// To prevent strange things like "103%" happening in the GUI, when a high multiplier is used --- by El Rapido
	
	int chrSkill = makeint(sti(chref.Experience.(expName))/500);

	if (CheckAttribute(chref,"skill."+expName+".bonus"))
	{
		skillExp = makeint(skillExp * stf(chref.skill.(expName).bonus));	// LDH added stf - 25Dec08
	}

	//-----------------------------------------------------------------------------------
	// Adjust the skill gained by character level, skill level and uneven gain in skills
	//-----------------------------------------------------------------------------------

	// Experience in each skill is worth more at low skill levels
	skillExp = skillExp * (11 - sti(chref.skill.(expName)));	// this produces 10,9,8,7,6,5,4,3,2,1

	// Adjust for uneven gain in skills
	int expMult = 1;
	if (_exp < 1000)
	{
		switch (expName)
		{
//		case "Leadership":	expMult =  3; break;	// LDH 30Apr09
		case "Fencing":		expMult =  3; break;
		case "Sailing":		expMult =  1; break;	// included for completeness
		case "Accuracy":	expMult =  1; break;	// included for completeness
		case "Cannons":		expMult =  1; break;	// included for completeness
		case "Repair":
			expMult = 50;							// repair is a lot of small skill gains, minimum of 20 sent by repair functions
			if (sti(chref.rank) < 3)		// keeps player from getting repair skill 5 or 6 in the tutorial - 25Feb09
			{
				expMult = 20;
			}
			break;
		case "Defence":		expMult =  6; break;
		case "Sneak":		expMult =  3; break;
		}
	}
	// Things we don't handle in the exp<1000 test above
	if (expName == "Leadership") expMult = 3;	// LDH moved outside of test - 30Apr09
	if (expName == "Grappling")  expMult = 4;	// grappling is always high experience but doesn't happen often. - 30Apr09
	if (expName == "Commerce") skillExp /= 2;	// commerce is usually high, we're cutting it in half here so above test is wrong for this

	skillExp *= expMult;

	// Level the skill gains across ranks. Experience is worth more at lower rank.
	// limit to 2 * experience - 01May09
	skillExp = makeint(makefloat(skillExp) * retMin(2.0, 4.0 / stf(chref.rank)) + 0.5);	// 2.0, 2.0, 1.3, 1.0, 0.75, 0.67 etc

	// If you think skills are gained too slow or too fast, here's where to tweak the overall rate.
	// Higher number gives faster gain for all skills.
	// float SkillTweak = 1.0;   						// changed by El Rapido to be configurable via internal settings
	float SkillTweak = SKILL_EXPERIENCE_MULTIPLIER;			// changed by El Rapido to be configurable via internal settings
	if (SkillTweak < 0.1) SkillTweak = 0.1;
	
	skillExp = makeint(makefloat(skillExp) * SkillTweak);

	int curSkill = sti(chref.Experience.(expName));
	if (_exp < 500)
		curSkill += skillExp;
	else
		curSkill += makeint(skillExp / 2.0);	// LDH this is from the old code and I'm not sure why it's used - 03Apr09

	//================================
	// Add the skill to the character
	//================================

	int newSkill = makeint(curSkill/500);
	if (LogsToggle == LOG_VERBOSE && IsInParty(GetMainCharacterIndex(), sti(chref.index)) == true && newSkill > chrSkill) Log_SetStringToLog(GetMySimpleName(chref) + " " + TranslateString("","Gained") + " +" + (newSkill - chrSkill) + "% " + XI_ConvertString("to_skill") + " " + XI_ConvertString(expName));

	// LDH traces 19Dec08
	int LowSkillTraceCutoff = 99999;	// 0 to trace all experience gains in compile log, higher to only see higher gains, 99999 to turn off traces
	if (IsMainCharacter(chref) && _exp >= LowSkillTraceCutoff)
	{
		string str;
		string tempExpName = StrLeft(expName + "          ", 10);	// formatting - pad with spaces to 10 characters
		str = "----- SKILLING ----- " + tempExpName + " (" + chref.skill.(expName) + ")  exp: " + realExp;
		if (newSkill > chrSkill && sti(chref.skill.(expName)) < 10)
			str += " +" + (newSkill-chrSkill) + "  " + chrSkill + " -> " + newSkill;
		Trace(str);
	}

	int nextSkill = 50000;
	while (curSkill >= nextSkill) // PB: Keep doing this while necessary
	{
		nextSkill = 50000; // PB: Reset
		if(curSkill==nextSkill) curSkill = 0;
		if(curSkill>nextSkill) curSkill = makeint(curSkill-nextSkill);
		if(sti(chref.skill.(expName)) < SKILL_MAX)
		{
			chref.skill.(expName) = sti(chref.skill.(expName)) + 1;
			if (iRealismMode == 0 && LogsToggle >= LOG_NORMAL && IsInParty(GetMainCharacterIndex(), sti(chref.index))) { // KK
				Log_SetStringToLog(GetMySimpleName(chref) + " " + TranslateString("","Gained") + " " + XI_ConvertString("skill_level") + " " + XI_ConvertString(expName) + " - " + sti(chref.skill.(expName)));
			}
		}
	}
	if (sti(chref.skill.(expName)) < SKILL_MAX)
		chref.Experience.(expName) = curSkill;
	else
		chref.Experience.(expName) = "0";*/

	//Levis removed this cause handled in exp function already.
	//============================================
	// This section handles character leveling up
	//============================================

	/*int retVal = false;		// set true if character levels up
	bool bShown = true;

	while (curExp >= nextExp) // PB: Keep doing this while necessary
	{
		if(CheckAttribute(chref,"perks.FreePoints"))
		{
			int freePerks = 0;
			int charPerks = 0;
			int chrPerksQ = 0;
			aref arPerksRoot; makearef(arPerksRoot,ChrPerksList.list);
			chrPerksQ = GetAttributesNum(arPerksRoot);
			for(int i=0; i<chrPerksQ; i++)
			{
				string perkName = GetAttributeName(GetAttributeN(arPerksRoot,i));
				if(!CheckCharacterPerk(chref,perkName)) freePerks = freePerks +1;
				if(CheckCharacterPerk(chref,perkName)) charPerks = charPerks +1;
			}
			if (freePerks > 0)
			{
				chref.perks.FreePoints = sti(chref.perks.FreePoints) + 1;
				if(!bAllies(chref))
				{
					if(CheckAttribute(chref,"chr_ai.type"))
					{
						if(chref.chr_ai.type!=LAI_TYPE_BUILDING)
						{
							if(CheckAttribute(chref,"donotshowXP")) { bShown = false; }
							if(sti(chref.perks.FreePoints)>0) { SetNextPerkToCharacter(chref, expName, bShown); }
						}
					}
				}
			}
			else
			{
				chref.perks.FreePoints = sti(chref.perks.FreePoints);
			}
		}
		else
		{
			chref.perks.FreePoints = 1;
		}

		chref.rank = sti(chref.rank) + 1;
		nextExp = CalculateExperienceFromRank(sti(chref.rank)+1);
		ResetHP(&chref);
		if(CheckAttribute(chref,"quest.officerprice"))
			chref.quest.officerprice = sti(chref.quest.officerprice) + 200 * OPRICE_LEVEL_MULT;
		retVal = true;
		if (IsInParty(GetMainCharacterIndex(), sti(chref.index))) { // KK
			Log_SetStringToLog(
				GetMySimpleName(chref) + " "
				+ TranslateString("","has gained a new level! (Now level")
				+ " " + chref.rank + ")");
			PlayStereoSound("interface\new_level.wav");
		}
		// LDH 01May09
		Trace(GetMySimpleName(chref) + " " + TranslateString("","has gained a new level! (Now level") + " " + chref.rank + ")");
	}

	if(retVal)
	{
		if(IsOfficer(chref) || IsCompanion(chref))
		{
			AddMsgToCharacter(chref,MSGICON_LEVELUP);
		}
	}*/
	//return retVal;
}

void Ship_AddCharacterExpChar(ref rCharacter, string expName, int iExpQuantity)
{
	AddXP(rCharacter, expName, iExpQuantity, XP_GROUP_PLAYER);
	/*bool bCompanion = IsCompanion(rCharacter);
	bool bNoShowXP;
	if (!bCompanion)
	{
		bNoShowXP = CheckAttribute(rCharacter,"donotshowXP");
		if (!bNoShowXP) { rCharacter.donotshowXP = true; }
	}
	rCharacter.tmpExp = true;//MAXIMUS: identifier for ships-owners. If NPC is fantom he will gain exp, but only during sea battle
	AddCharacterExpChar(rCharacter, expName, makeint(iExpQuantity));
	if(!bCompanion && !bNoShowXP) DeleteAttribute(rCharacter,"donotshowXP");

	if(IsMainCharacter(rCharacter))
	{
		fSeaExp = fSeaExp + iExpQuantity;
	}*/
}

void AddExpAndShow(ref chref, int _exp)
{
	AddXP(chref, "", _exp, XP_GROUP_PLAYER);
	Log_SetStringToLog("+" + _exp + XI_ConvertString("add experience"));
	//AddCharacterExp(chref, _exp);
}

// NK -->
int AddCharacterExp(ref chref,int _exp)
{
	return makeint(AddXP(chref, "", _exp, XP_GROUP_OFFIC));
	/*ref chref;
	int i, cn, myindex;
	myindex = sti(chref.index);
	makeref(chref, chref);
	AddCharacterExpNS(chref, _exp);
	if(IsOfficer(chref)) chref = GetMainCharacter();
	float sxmult = 1.0 + GetOfficersPerkUsing(chref,"SharedExperience");
	//Log_SetStringToLog("Mult " + sxmult);
	//if(myindex != sti(chref.index)) AddCharacterExpNS(chref, _exp*0.5*sxmult);
	for(i=0; i <= GetPassengersQuantity(chref); i++)
	{
		cn = GetPassenger(chref, i);
		if(cn == -1 || myindex == cn) continue;
		if(CheckAttribute(Characters[cn],"prisoned") && sti(Characters[cn].prisoned)) continue;
		if(CheckAttribute(Characters[cn],"officerboss"))
		{
			Log_SetStringToLog(Characters[cn].name + " " + Characters[cn].lastname + " is officer");
			AddCharacterExpNS(Characters[cn], _exp * 0.5 * sxmult);
		}
		else
		{
			Log_SetStringToLog(Characters[cn].name + " " + Characters[cn].lastname + " is just passenger");
			AddCharacterExpNS(Characters[cn], _exp * 0.25 * sxmult);
		}
	}*/
	//Log_SetStringToLog("1");
	/*for(i=0; i < 4; i++)
	{
		cn = GetOfficersIndex(chref, i);
		if(cn == -1) { /*Log_SetStringToLog("off " + i + " not"); continue; }
		if(cn == myindex) { /*Log_SetStringToLog("off " + i + " already"); continue; }
		//Log_SetStringToLog(Characters[cn].name + " " + Characters[cn].lastname + " is officer, getting " + makeint(makefloat(_exp) * 0.5 * sxmult));
		AddCharacterExpNS(&Characters[cn], makeint(makefloat(_exp) * 0.5 * sxmult));
	}
	if(!CheckAttribute(chref,"Experience")) chref.Experience = 1; // MAXIMUS 28.07.2006
	return sti(chref.Experience);*/
}

int AddCharacterExpNS(ref chref,int _exp)
{
	return makeint(AddXP(chref, "", _exp, XP_GROUP_PLAYER));
	/*//int tmpLangFileID = LanguageOpenFile("interface_strings.txt");
	if(_exp < 1) return 0;
	int retVal = false;
	//if(CheckCharacterPerk(chref,"SharedExperience")) _exp *= 1.25;
	// PB -->
	bool ShowXP;
	switch(LogsToggle)
	{
		case LOG_QUIET:
			ShowXP = false;
		break;
		case LOG_LACONIC:
			ShowXP = IsMainCharacter(chref);
		break;
		case LOG_NORMAL:
			ShowXP = IsMainCharacter(chref) || IsOfficer(chref) || IsCompanion(chref);
		break;
		case LOG_VERBOSE:
			ShowXP = true;
		break;
	}
	// PB <--
	if(ShowXP && _exp > 10 * makeint(chref.rank) && _exp > 25 && !CheckAttribute(chref,"donotshowXP")) { Log_SetStringToLog(chref.name + " " + chref.lastname + " " + TranslateString("","Gained") + " " + makeint(_exp) + " " + XI_ConvertString("XP")); }
	if(!CheckAttribute(chref,"Experience")) chref.Experience = 1; // MAXIMUS 28.07.2006
	int curExp = sti(chref.Experience) + makeint(_exp);
	chref.Experience = curExp;
	int nextExp = CalculateExperienceFromRank(sti(chref.rank)+1);
	
	//NEED TO USE THIS
	
	/*while (curExp >= nextExp) // PB: Keep doing this while necessary
	{
		if(CheckAttribute(chref,"perks.FreePoints"))
		{
			int freePerks = 0;
			int charPerks = 0;
			int chrPerksQ = 0;
			aref arPerksRoot; makearef(arPerksRoot,ChrPerksList.list);
			chrPerksQ = GetAttributesNum(arPerksRoot);
			for(int i=0; i<chrPerksQ; i++)
			{
				string perkName = GetAttributeName(GetAttributeN(arPerksRoot,i));
				if(!CheckCharacterPerk(chref,perkName)) freePerks = freePerks +1;
				if(CheckCharacterPerk(chref,perkName)) charPerks = charPerks +1;
			}
			if (freePerks > 0)
			{
				chref.perks.FreePoints = sti(chref.perks.FreePoints) + 1;
				if(!bAllies(chref))
				{
					if(CheckAttribute(chref,"chr_ai.type"))
					{
						if(chref.chr_ai.type!=LAI_TYPE_BUILDING)
						{
							if(CheckAttribute(chref,"donotshowXP")) { bShown = false; }
							if(sti(chref.perks.FreePoints)>0) { SetNextPerkToCharacter(chref, expName, bShown); }
						}
					}
				}
			}
			else
			{
				chref.perks.FreePoints = sti(chref.perks.FreePoints);
			}
		}
		else
		{
			chref.perks.FreePoints = 1;
		}

		chref.rank = sti(chref.rank) + 1;
		nextExp = CalculateExperienceFromRank(sti(chref.rank)+1);
		ResetHP(&chref);
		if(CheckAttribute(chref,"quest.officerprice"))
			chref.quest.officerprice = sti(chref.quest.officerprice) + 200 * OPRICE_LEVEL_MULT;
		retVal = true;
		if (IsInParty(GetMainCharacterIndex(), sti(chref.index))) { // KK
			Log_SetStringToLog(
				GetMySimpleName(chref) + " "
				+ TranslateString("","has gained a new level! (Now level")
				+ " " + chref.rank + ")");
			PlayStereoSound("interface\new_level.wav");
		}
		// LDH 01May09
		Trace(GetMySimpleName(chref) + " " + TranslateString("","has gained a new level! (Now level") + " " + chref.rank + ")");
	}
	
	while(curExp>=nextExp)
	{
		chref.rank = sti(chref.rank) + 1;
		chref.Skill.FreeSkill = sti(chref.Skill.FreeSkill) + ADD_SKILLPOINTS_PERLEVEL; // TIH changed to user configurable setting Nov14'06
		if(CheckAttribute(chref,"perks.FreePoints"))
		{	chref.perks.FreePoints = sti(chref.perks.FreePoints) + 1;
		}
		else
		{	chref.perks.FreePoints = 1;
		}
		nextExp = CalculateExperienceFromRank(sti(chref.rank)+1);
		ResetHP(&chref); // NK 05-07-03 -
		/*float mhp = LAi_GetCharacterMaxHP(chref) + 5.0;
		if(CheckAttribute(chref,"officer")) mhp = LAi_GetCharacterMaxHP(chref) + makeint(11-GetDifficulty());
		LAi_SetHP(chref,LAi_GetCharacterHP(chref),mhp);
		if(CheckAttribute(chref,"quest.officerprice")) chref.quest.officerprice = sti(chref.quest.officerprice) + 200 * OPRICE_LEVEL_MULT;
		if (GetAttribute(chref, "location") != "Tutorial_Deck") // PB: Skip update here
		{
			retVal = true;
			Log_SetStringToLog(chref.name + " " + chref.lastname + " " + TranslateString("","has gained a new level! (Now level") + " " + chref.rank + ")");
			PlayStereoSound("interface\new_level.wav");
		}
	}
	
	if(retVal)
	{
		if(IsOfficer(chref) || IsCompanion(chref))
		{
			AddMsgToCharacter(chref,MSGICON_LEVELUP);
		}
	}
	
	return retVal;
	//LanguageCloseFile(tmpLangFileID);*/
}
// NK <--

//----------------------------------------------------------------------------------------------//
//Functions to handle XP																		//
//----------------------------------------------------------------------------------------------//

int CalculateExperienceFromRank(int _Rank)
{
	if(_Rank < 1) return 0;
	/*int oldxp = CalculateExperienceFromRank((_Rank-1));
	float retVal = sqrt(makefloat(_Rank)) * 400.0 + makefloat(_Rank)*100.0;
	//int retVal = (MakeInt(1000 * pow(_Rank,2.3))/1000)*1000;
	return oldxp + makeint(retVal);*/
	return makeint(1200*pow((_Rank-1),2)+100*(_Rank-1)+800*pow(2.75,sqrt(_Rank-1)));
}

//----------------------------------------------------------------------------------------------//
//Functions to handle skill gain																//
//----------------------------------------------------------------------------------------------//

void InitAutoSkillsSystem(ref refCharacter, bool dailyUpdate)
{
	if(!IsMainCharacter(refCharacter) && !sti(refCharacter.index)==GetMainCharacterIndex() && !bAllies(refCharacter) && !CheckAttribute(refCharacter, "IsFantom"))
	{
		if(DEBUG_EXPERIENCE>1) Trace("XP LOG: Start InitAutoSkillsSystem");
		if (CheckAttribute(refCharacter,"corpse")) return;
		if (!CheckAttribute(refCharacter,"skillsetup")) InitCharacterSkills(refCharacter);
		/*if (!CheckAttribute(refCharacter,"Experience")) refCharacter.Experience = 1;
		if (!CheckAttribute(refCharacter,"rank")) refCharacter.rank = 1;*/ //Should be fixed by InitChracter Already
		//int nextSkill = 50000;
		//int skillsCalc = 0;
		//if(SKILLS_UP_BONUS<10) SKILLS_UP_BONUS = 10;
		//if(SKILLS_UP_BONUS>99) SKILLS_UP_BONUS = 99;
		float curSpeed = makefloat(stf(SKILLS_UP_BONUS+100)/100);
		int numUp = sti(GetDataYear() + GetDataMonth() + AllDaysCount);
		int expUp = sti(GetDifficulty()*500);
		if (CheckAttribute(refCharacter,"locDate") && numUp - sti(refCharacter.locDate) > 0) {
			numUp = sti(numUp - sti(refCharacter.locDate));
			expUp = expUp*numUp;
		} else {
			expUp = 0;
		}
		//Levis: The rest will now all be handled by the addxp function.
		AddXP(refCharacter, "", expUp, XP_GROUP_PLAYER);
		refCharacter.locDate = sti(GetDataYear() + GetDataMonth() + AllDaysCount);
	}
//MAXIMUS: <--
	/*int bonusCalc = 0;
	int randBonus = -1;
	for (int c = 0; c < 10; c++)
	{
		if (CheckAttribute(refCharacter,"bonusCalc"))
			bonusCalc = sti(refCharacter.bonusCalc);
		else
			refCharacter.bonusCalc = bonusCalc;
		string curSkillName = GetSkillName(c);
		if (!CheckAttribute(refCharacter, "Experience." + curSkillName))
			refCharacter.experience.(curSkillName) = makeint(makeint(Rand(19)+1)*3000));
		else
			refCharacter.experience.(curSkillName) = makeint(sti(refCharacter.experience.(curSkillName)) + expUp); //MAXIMUS: skills increase automatically, depending on days counting

		if (!CheckAttribute(refCharacter, "skill." + curSkillName) || sti(refCharacter.skill.(curSkillName)) == 0) refCharacter.skill.(curSkillName) = 1;
		if (sti(refCharacter.Experience.(curSkillName))>=nextSkill) {
			if(CheckAttribute(refCharacter,"skill."+curSkillName))
			{
				if(sti(refCharacter.skill.(curSkillName))< SKILL_MAX)
				{
					refCharacter.skill.(curSkillName) = sti(refCharacter.skill.(curSkillName)) + 1;
					refCharacter.Experience.(curSkillName) = makeint(sti(refCharacter.Experience.(curSkillName))-nextSkill);
				}
				else
				{
					// Aconcagua: what should this have done???
					// refCharacter.skill.(curSkillName) = sti(refCharacter.skill.(curSkillName));
					refCharacter.Experience.(curSkillName) = "0";
				}
			}
		}
		if(!CheckAttribute(refCharacter,"perks.FreePoints")) { refCharacter.perks.FreePoints = 1; }
		if(GetCharacterindex(refCharacter.id)==GetMainCharacterIndex()) continue;
		if(bAllies(refCharacter)) continue;
		if(CheckAttribute(refCharacter,"chr_ai.type"))
		{
			if(refCharacter.chr_ai.type != LAI_TYPE_BUILDING)
			{
				if(!CheckAttribute(refCharacter,"quest.officerprice"))
				{
					if(GetLevel(refCharacter)>=3)
					{
						int freePerks = 0;
						int usedPerks = 0;
						aref arPerksRoot; makearef(arPerksRoot,ChrPerksList.list);
						for (int i = 0; i < GetAttributesNum(arPerksRoot); i++) {
							string perkName = GetAttributeName(GetAttributeN(arPerksRoot,i));
							if (!CheckCharacterPerk(refCharacter, perkName))
								freePerks++;
							else
								usedPerks++;
						}
						if (freePerks >= 3) {
							if (sti(refCharacter.perks.FreePoints) == 0) refCharacter.perks.FreePoints = makeint(Rand(2) + 1);
						}
					}
					if (CheckAttribute(refCharacter, "perks.FreePoints") && sti(refCharacter.perks.FreePoints) > 0) SetNextPerkToCharacter(refCharacter, curSkillName, false);
				}
			}
		}
		refCharacter.autoskills = 1;
		if(sti(refCharacter.skill.(curSkillName))>1) { skillsCalc = skillsCalc + sti(refCharacter.skill.(curSkillName)) - 1; }
		if(sti(refCharacter.bonusCalc)<2 && randBonus==-1) { randBonus = Rand(9); }
		if(sti(refCharacter.bonusCalc)<2 && CheckAttribute(refCharacter,"skill."+curSkillName+".bonus") && stf(refCharacter.skill.(curSkillName).bonus) > 1.0)
		{
			refCharacter.bonusCalc = sti(refCharacter.bonusCalc)+1;
			if(randBonus==c) {
				randBonus = c-1;
				// Aconcagua:
				if(randBonus < 0)
				{
					randBonus = 9;
				}
			}
		}
	}
	if (sti(refCharacter.bonusCalc) < 2 && randBonus >= 0) {
		string bonusSkillName = GetSkillName(randBonus);
		refCharacter.skill.(bonusSkillName).bonus = curSpeed;
		bonusSkillName = GetRandSkillName(refCharacter);
		refCharacter.skill.(bonusSkillName).bonus = curSpeed;
	}
	if (!CheckAttribute(refCharacter,"Money")) {
		refCharacter.Money = GetRandCharMoney(refCharacter, makeint(Rand(15)+10));
	} else {
		if (sti(refCharacter.Money) == 0) refCharacter.Money = GetRandCharMoney(refCharacter, makeint(Rand(15)+10));
	}
	skillsCalc = makeint(skillsCalc / 2);
	refCharacter.rank = sti(1 + skillsCalc);
	refCharacter.Experience = CalculateExperienceFromRank(sti(refCharacter.rank));
	if (!CheckAttribute(refCharacter, "quest.officerprice")) {
		LAi_SetCurHPMax(refCharacter);
		ResetHP(refCharacter);
	}*/
}

string GetRandSkillName(ref chr)
{
	string SkillName = "";
	int skillNum = Rand(9);
	for(int c = 0; c < 10; c++)
	{
		string curSkill = GetSkillName(c);
		if(skillNum==c)
		{
			if(!CheckAttribute(chr,"skill."+curSkill+".bonus")) { SkillName = curSkill; }
			else
			{
				if(skillNum<=3) { skillNum = skillNum+sti(Rand(2)+1); }
				else { if(skillNum<=9) { skillNum = skillNum-sti(Rand(8)+1); } }
				SkillName = GetSkillName(skillNum);
			}
			return SkillName;
		}
	}
	return "Leadership";
}

/*void DailySkillsUpdate()
{
	for(int i=0; i<CHARACTERS_QUANTITY; i++)
	{
		ref curChar = GetCharacter(i);
		// LDH simplified - only difference is that passengers will be excluded - 03Apr09
		// bAllies is tested before this function is called, so this is redundant
		if (!bAllies(curChar))
		{
			if(CheckAttribute(curChar,"skill") && CheckAttribute(curChar,"sex") && CheckAttribute(curChar,"chr_ai.type"))
			{
				if(curChar.chr_ai.type!=LAI_TYPE_BUILDING)
				{
					InitAutoSkillsSystem(curChar, true);
				}
			}
		}
	}

}*/

//----------------------------------------------------------------------------------------------//
//Functions to handle skills																	//
//----------------------------------------------------------------------------------------------//

int GetSummonSkillFromName(ref chref, string skillName)
{
	int skillPoints = CalcSummOnSkillFromName(chref, skillName);
	return ApplyFleetMali(chref, skillName, skillPoints, true);
}

int CalcSummOnSkillFromName(ref chref, string skillName)
{
	int sumSkill = CalcEffectiveSkill(chref, skillName);

	// Aconcagua: Fencing is a personal skill...
	// TODO: sneak SEEMS to be so, to - verify!
	if(skillName == SKILL_FENCING || skillName == SKILL_SNEAK)
	{
		return sumSkill;
	}

	int i, cn, curSkill, skillPoints;
	if(IsMainCharacter(chref))
	{
		int num = GetPassengersQuantity(chref);
		for(i = 0; i < num; i++)
		{
			cn = GetPassenger(chref, i);
			if(cn < 0)
				continue;

			ref chr = GetCharacter(cn);
			// IsTrader == true => IsCompanion == true !
			if(CheckAttribute(chr, "prisoned") || IsCompanion(chr))// || IsTrader(chr))
				continue;

			skillPoints = CalcEffectiveSkill(chr, skillName);
			if(skillPoints > sumSkill)
				sumSkill = skillPoints;
		}
	}
	else
	{
		for(i = 1; i < 4; i++)
		{
			cn = GetOfficersIndex(&chref,i);
			if(cn >= 0)
			{
				skillPoints = CalcEffectiveSkill(GetCharacter(cn), skillName);
				if(skillPoints > sumSkill)
					sumSkill = skillPoints;
			}
		}
	}
	return sumSkill;
}

// NK -->
int GetAverageSkill(ref chr, string skl)
{
	return (GetEffectiveSkill(&chr, skl) + GetSummonSkillFromName(&chr, skl))/2;
}

int GetSkillBool(ref chr, string skill, bool summon) //will return summon if bool summon == true, else return normal GCS.
{
	if(summon) return GetSummonSkillFromName(&chr, skill);
	return GetEffectiveSkill(&chr, skill);
}
// NK <--

int GetSummonSkill(ref chref,int _SkillType)
{
	return GetSummonSkillFromName(&chref,GetSkillName(_SkillType));
}

string GetBasicSkill(ref chref)
{
	aref refSkill;
	makearef(refSkill,chref.Skill);
	string skillName;

	int i=0;
	int maxSkillValue = -1;
	string maxSkillName;
	for(i=0; i<10; i++)
	{
		skillName = GetSkillName(i);
		if(MakeInt(refSkill.(skillName))>maxSkillValue)
		{
			maxSkillName = skillName;
			maxSkillValue = MakeInt(refSkill.(skillName));
		}
	}
	return maxSkillName;
}

//----------------------------------------------------------------------------------------------//
//Functions to handle HP Bonus																	//
//----------------------------------------------------------------------------------------------//

float ResetHP(ref chr)
{
	float HP = CHAR_START_HITPOINTS;
	float level = makefloat(GetLevel(&chr));
	if (level == 0) level = 20;	// LDH 08Feb09
	if(CheckAttribute(chr, "HPBonus")) HP += stf(chr.HPBonus);
	HP += (level - 1) * CHAR_HITPOINTS_PER_LEVEL;
// LDH --> Added toughness perk from elrapido verson of ResetMaxHP
	if(IsCharacterPerkOn(chr, "Toughness"))
	{
		HP = HP * 1.1;			// 10% health bonus for the tough guys
		chr.chr_ai.hp_dlt = LAI_DEFAULT_DLTHP * TOUGHNESS_REGEN_MULT; // higher regeneration rate, default is 0.1 - LDH 14Apr09
	}
// LDH <--
	chr.chr_ai.hp = HP;
	chr.chr_ai.hp_max = HP;
	chr.old.chr_ai.hp = HP;
	chr.old.chr_ai.hp_max = HP;
	return HP;
}

int ChangeHPBonus(ref chr, int offset)
{
	if(CheckAttribute(chr, "HPBonus")) offset += sti(chr.HPBonus);
	chr.HPBonus = offset;
	return sti(offset);
}

//----------------------------------------------------------------------------------------------//
//Functions to handle perks																		//
//----------------------------------------------------------------------------------------------//

string charInfo, tempInfo, perksInfo;

void SetNextPerkToCharacter(ref refCharacter, string skillName, bool bShown)
{
	int tmpLangFileID = LanguageOpenFile("AbilityDescribe.txt");
	//int tmpTextFileID = LanguageOpenFile("interface_strings.txt");
	int usedPerk = 0;
	charInfo = GetMyFullName(refCharacter) + " " + TranslateString("","GainedPerk") + " ";
	tempInfo = "";
	perksInfo = "";
	if(skillName==SKILL_DEFENCE) { if(SendMessage(GetMainCharacter(), "ls", MSG_CHARACTER_EX_MSG, "IsFightMode")!=0) skillName = SKILL_FENCING; }
	if(skillName==SKILL_DEFENCE) { if(CheckAttribute(objLandInterface,"data.riskAlarm") && objLandInterface.data.riskAlarm==1) skillName = SKILL_FENCING; }
	if(sti(refCharacter.perks.FreePoints)>0)
	{
		switch(skillName)
		{
			case SKILL_LEADERSHIP: if(SetPerks_Leadership(refCharacter, true)>0) { usedPerk = SetPerks_Leadership(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); } break;
			case SKILL_FENCING: if(SetPerks_Blade(refCharacter, true)>0) { usedPerk = SetPerks_Blade(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); } break;
			case SKILL_SAILING: if(SetPerks_Navigator(refCharacter, true)>0) { usedPerk = SetPerks_Navigator(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); } break;
			case SKILL_ACCURACY: if(SetPerks_Gun(refCharacter, true)>0) { usedPerk = SetPerks_Gun(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); } break;
			case SKILL_CANNONS: if(SetPerks_Cannon(refCharacter, true)>0) { usedPerk = SetPerks_Cannon(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); } break;
			case SKILL_GRAPPLING: if(SetPerks_Grappling(refCharacter, true)>0) { usedPerk = SetPerks_Grappling(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); } break;
			case SKILL_REPAIR: if(SetPerks_QMaster(refCharacter, true)>0) { usedPerk = SetPerks_QMaster(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); } break;
			case SKILL_DEFENCE: if(SetPerks_Defence(refCharacter, true)>0) { usedPerk = SetPerks_Defence(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); } break;
			case SKILL_COMMERCE: if(SetPerks_QMaster(refCharacter, true)>0) { usedPerk = SetPerks_QMaster(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); } break;
			case SKILL_SNEAK: if(SetPerks_Leadership(refCharacter, true)>0) { usedPerk = SetPerks_Leadership(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); } break;
		}
		if (iRealismMode == 0 || IsMainCharacter(refCharacter) || IsInParty(GetMainCharacterIndex(), sti(refCharacter.index))) { // KK
			if (usedPerk > 0 && LogsToggle >= LOG_NORMAL && bShown == true) Log_SetStringToLog(charInfo + tempInfo);
		}
		LanguageCloseFile(tmpLangFileID);
		//LanguageCloseFile(tmpTextFileID);
	}
	if(usedPerk<=0) { ResortFreePerks(refCharacter); }
}

void ResortFreePerks(ref refCharacter)
{
	int tmpLangFileID = LanguageOpenFile("AbilityDescribe.txt");
	//int tmpTextFileID = LanguageOpenFile("interface_strings.txt");
	int usedPerk = 0;
	charInfo = GetMySimpleName(refCharacter) + " " + TranslateString("","GainedPerk") + " ";
	if(sti(refCharacter.perks.FreePoints)>0)
	{
		if(SetPerks_Blade(refCharacter, true)>0) { usedPerk = SetPerks_Blade(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); }
		else
		{
			if(SetPerks_Gun(refCharacter, true)>0) { usedPerk = SetPerks_Gun(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); }
			else
			{
				if(SetPerks_Leadership(refCharacter, true)>0) { usedPerk = SetPerks_Leadership(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); }
				else
				{
					if(SetPerks_Navigator(refCharacter, true)>0) { usedPerk = SetPerks_Navigator(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); }
					else
					{
						if(SetPerks_Cannon(refCharacter, true)>0) { usedPerk = SetPerks_Cannon(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); }
						else
						{
							if(SetPerks_Grappling(refCharacter, true)>0) { usedPerk = SetPerks_Grappling(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); }
							else
							{
								if(SetPerks_Defence(refCharacter, true)>0) { usedPerk = SetPerks_Defence(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); }
								else
								{
									if(SetPerks_QMaster(refCharacter, true)>0) { usedPerk = SetPerks_QMaster(refCharacter, false); tempInfo = LanguageConvertString(tmpLangFileID,perksInfo); }
								}
							}
						}
					}
				}
			}
		}
	}
	LanguageCloseFile(tmpLangFileID);
	//LanguageCloseFile(tmpTextFileID);
}

int SetPerks_Leadership(ref Npchar, bool bCheck)
{
	int usedPerk = 0;
	if(!CheckAttribute(Npchar,"Perks.List.SharedExperience")) { perksInfo = "SharedExperience"; if(!bCheck) { Npchar.Perks.List.SharedExperience = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	else
	{
		if(!CheckAttribute(Npchar,"Perks.List.IronWill")) { perksInfo = "IronWill"; if(!bCheck) { Npchar.Perks.List.IronWill = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
		else { return 0; }
	}

	return usedPerk;
}

int SetPerks_Blade(ref Npchar, bool bCheck)
{
	int usedPerk = 0;
	if(!CheckAttribute(Npchar,"Perks.List.BasicDefence")) { perksInfo = "BasicDefence"; if(!bCheck) { Npchar.Perks.List.BasicDefence = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	else
	{
		if(!CheckAttribute(Npchar,"Perks.List.AdvancedDefence")) { perksInfo = "AdvancedDefence"; if(!bCheck) { Npchar.Perks.List.AdvancedDefence = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
		else
		{
			if(!CheckAttribute(Npchar,"Perks.List.Toughness")) { perksInfo = "Toughness"; if(!bCheck) { Npchar.Perks.List.Toughness = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
			if(!CheckAttribute(Npchar,"Perks.List.Rush")) { perksInfo = "Rush"; if(!bCheck) { Npchar.Perks.List.Rush = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
			if(!CheckAttribute(Npchar,"Perks.List.CriticalHit")) { perksInfo = "CriticalHit"; if(!bCheck) { Npchar.Perks.List.CriticalHit = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
			else
			{
				if(!CheckAttribute(Npchar,"Perks.List.SwordplayProfessional")) { perksInfo = "SwordplayProfessional"; if(!bCheck) { Npchar.Perks.List.SwordplayProfessional = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
				else { return 0; }
			}
		}
	}

	return usedPerk;
}

int SetPerks_Gun(ref Npchar, bool bCheck)
{
	int usedPerk = 0;
	if(!CheckAttribute(Npchar,"Perks.List.Gunman")) { perksInfo = "Gunman"; if(!bCheck) { Npchar.Perks.List.Gunman = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	else
	{
		if(!CheckAttribute(Npchar,"Perks.List.GunProfessional")) { perksInfo = "GunProfessional"; if(!bCheck) { Npchar.Perks.List.GunProfessional = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
		else
		{
			if(!CheckAttribute(Npchar,"Perks.List.GunFighter")) { perksInfo = "GunFighter"; if(!bCheck) { Npchar.Perks.List.GunFighter = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }	//JRH
		}
		else { return 0; }
	}

	return usedPerk;
}

int SetPerks_Cannon(ref Npchar, bool bCheck)
{
	int usedPerk = 0;
	if(!CheckAttribute(Npchar,"Perks.List.FastReload")) { perksInfo = "FastReload"; if(!bCheck) { Npchar.Perks.List.FastReload = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	else
	{
		if(!CheckAttribute(Npchar,"Perks.List.ImmediateReload")) { perksInfo = "ImmediateReload"; if(!bCheck) { Npchar.Perks.List.ImmediateReload = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	}

	if(!CheckAttribute(Npchar,"Perks.List.HullDamageUp")) { perksInfo = "HullDamageUp"; if(!bCheck) { Npchar.Perks.List.HullDamageUp = "1"; } usedPerk = 1; return usedPerk; }
	if(!CheckAttribute(Npchar,"Perks.List.SailsDamageUp")) { perksInfo = "SailsDamageUp"; if(!bCheck) { Npchar.Perks.List.SailsDamageUp = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	if(!CheckAttribute(Npchar,"Perks.List.CrewDamageUp")) { perksInfo = "CrewDamageUp"; if(!bCheck) { Npchar.Perks.List.CrewDamageUp ="1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	if(!CheckAttribute(Npchar,"Perks.List.perkLongRangeGrappling")) { perksInfo = "LongRangeGrappling"; if(!bCheck) { Npchar.Perks.List.perkLongRangeGrappling ="1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }

	if(sti(Npchar.Perks.List.HullDamageUp)+sti(Npchar.Perks.List.SailsDamageUp)+sti(Npchar.Perks.List.CrewDamageUp)>2)
	{
		if(!CheckAttribute(Npchar,"Perks.List.CriticalShoot")) { perksInfo = "CriticalShoot"; if(!bCheck) { Npchar.Perks.List.CriticalShoot = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
		else
		{
			if(!CheckAttribute(Npchar,"Perks.List.LongRangeShoot")) { perksInfo = "LongRangeShoot"; if(!bCheck) { Npchar.Perks.List.LongRangeShoot ="1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
			else
			{
				if(!CheckAttribute(Npchar,"Perks.List.CannonProfessional")) { perksInfo = "CannonProfessional"; if(!bCheck) { Npchar.Perks.List.CannonProfessional ="1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
				else { return 0; }
			}
		}
	}

	return usedPerk;
}

int SetPerks_Navigator(ref Npchar, bool bCheck)
{
	int usedPerk = 0;
	if(!CheckAttribute(Npchar,"Perks.List.ShipSpeedUp")) { perksInfo = "ShipSpeedUp"; if(!bCheck) { Npchar.Perks.List.ShipSpeedUp = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	else
	{
		if(!CheckAttribute(Npchar,"Perks.List.ShipTurnRateUp")) { perksInfo = "ShipTurnRateUp"; if(!bCheck) { Npchar.Perks.List.ShipTurnRateUp = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
		else
		{
			if(!CheckAttribute(Npchar,"Perks.List.Turn180")) { perksInfo = "Turn180"; if(!bCheck) { Npchar.Perks.List.Turn180 = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
			else
			{
				if(!CheckAttribute(Npchar,"Perks.List.SandbankManeuver")) { perksInfo = "SandbankManeuver"; if(!bCheck) { Npchar.Perks.List.SandbankManeuver = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
				else
				{
					if(!CheckAttribute(Npchar,"Perks.List.StormProfessional")) { perksInfo = "StormProfessional"; if(!bCheck) { Npchar.Perks.List.StormProfessional = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
					else
					{
						if(!CheckAttribute(Npchar,"Perks.List.SailingProfessional")) { perksInfo = "SailingProfessional"; if(!bCheck) { Npchar.Perks.List.SailingProfessional = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
						else { return 0; }
					}
				}
			}
		}
	}

	return usedPerk;
}

int SetPerks_Grappling(ref Npchar, bool bCheck)
{
	int usedPerk = 0;
	bool tmp = false;
	if(!CheckAttribute(Npchar,"Perks.List.LongRangeGrappling")) tmp = true;
	else { if(Npchar.Perks.List.LongRangeGrappling != "1") tmp = true;}
	if(tmp) { perksInfo = "LongRangeGrappling"; if(!bCheck) { Npchar.Perks.List.LongRangeGrappling = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	else
	{
		tmp = false;
		if(!CheckAttribute(Npchar,"Perks.List.GrapplingProfessional")) tmp = true;
		else { if(Npchar.Perks.List.GrapplingProfessional != "1") tmp = true; }
		if(tmp) { perksInfo = "GrapplingProfessional"; if(!bCheck) { Npchar.Perks.List.GrapplingProfessional = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
		else
		{
			if(!CheckAttribute(Npchar,"Perks.List.Brander")) { perksInfo = "Brander"; if(!bCheck) { Npchar.Perks.List.Brander = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
			else
			{
				if(!CheckAttribute(Npchar,"Perks.List.Musketsshoot")) { perksInfo = "Musketsshoot"; if(!bCheck) { Npchar.Perks.List.Musketsshoot = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
				else
				{
					if(!CheckAttribute(Npchar,"Perks.List.Instantboarding")) { perksInfo = "Instantboarding"; if(!bCheck) { Npchar.Perks.List.Instantboarding = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
					else
					{
						if(!CheckAttribute(Npchar,"Perks.List.Troopers")) { perksInfo = "Troopers"; if(!bCheck) { Npchar.Perks.List.Troopers = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
						else { return 0; }
					}
				}
			}
		}
	}

	return usedPerk;
}

int SetPerks_Defence(ref Npchar, bool bCheck)
{
	int usedPerk = 0;
	bool tmp = false;
	if(!CheckAttribute(Npchar,"Perks.List.BasicBattleState")) tmp = true;
	else { if(Npchar.Perks.List.BasicBattleState != "1") tmp = true;}
	if(tmp) { perksInfo = "BasicBattleState"; if(!bCheck) { Npchar.Perks.List.BasicBattleState = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	else
	{
		tmp = false;
		if(!CheckAttribute(Npchar,"Perks.List.AdvancedBattleState")) tmp = true;
		else { if(Npchar.Perks.List.AdvancedBattleState != "1") tmp = true;}
		if(tmp) { perksInfo = "AdvancedBattleState"; if(!bCheck) { Npchar.Perks.List.AdvancedBattleState = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
		else
		{
			tmp = false;
			if(!CheckAttribute(Npchar,"Perks.List.ShipDefenceProfessional")) tmp = true;
			else { if(Npchar.Perks.List.ShipDefenceProfessional != "1") tmp = true;}
			if(tmp) { perksInfo = "ShipDefenceProfessional"; if(!bCheck) { Npchar.Perks.List.ShipDefenceProfessional = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
			else { return 0; }
		}
	}

	return usedPerk;
}

int SetPerks_QMaster(ref Npchar, bool bCheck)
{
	int usedPerk = 0;
	if(!CheckAttribute(Npchar,"Perks.List.LightRepair")) { perksInfo = "LightRepair"; if(!bCheck) { Npchar.Perks.List.LightRepair = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	else
	{
		if(!CheckAttribute(Npchar,"Perks.List.InstantRepair")) { perksInfo = "InstantRepair"; if(!bCheck) { Npchar.Perks.List.InstantRepair = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	}
	if(!CheckAttribute(Npchar,"Perks.List.Trustworthy")) { perksInfo = "Trustworthy"; if(!bCheck) { Npchar.Perks.List.Trustworthy = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	if(!CheckAttribute(Npchar,"Perks.List.BasicCommerce")) { perksInfo = "BasicCommerce"; if(!bCheck) { Npchar.Perks.List.BasicCommerce = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
	else
	{
		if(!CheckAttribute(Npchar,"Perks.List.AdvancedCommerce")) { perksInfo = "AdvancedCommerce"; if(!bCheck) { Npchar.Perks.List.AdvancedCommerce = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
		else
		{
			if(!CheckAttribute(Npchar,"Perks.List.BasicLandOwner")) { perksInfo = "BasicLandOwner"; if(!bCheck) { Npchar.Perks.List.BasicLandOwner = "1"; Npchar.perks.FreePoints = sti(Npchar.perks.FreePoints) - 1; } usedPerk = 1; return usedPerk; }
			else { return 0; }
		}
	}

	return usedPerk;
}